#define TYPE_MEM		0	/* Memory manager */
#define TYPE_DRI		1	/* Direct render agent */
#define TYPE_FB0		2	/* Frame buffer head 0 */
#define TYPE_FB1		3	/* Frame buffer head 1 */
#define TYPE_FB2		4	/* Frame buffer head 2 */
#define TYPE_FB3		5	/* Frame buffer head 3 */
#define TYPE_LAST		5
#define NUM_TYPES		6

struct vga_shared {
	struct vga_dev *device[NUM_TYPES];
	struct semaphore shared_sem;
	int count;			/* Devices */
	int users;			/* Active users */
	struct semaphore fb_sem;
	void (*lock_release)(struct vga_dev *, void *);
	void *lock_context;
	struct vga_driver *lock_owner;
};

struct vga_dev {
	struct list_head next;		/* All VGA devices */
	struct list_head router_list;	/* By VGA router (not yet done) */
	struct vga_driver *driver;
	struct pci_dev *pci_dev;
	struct device dev;
	int unit;
	struct vga_shared *shared;
};

struct vga_device_id {
	__u32 vendor, device;		/* Vendor and device ID or PCI_ANY_ID*/
	__u32 subvendor, subdevice;	/* Subsystem ID's or PCI_ANY_ID */
	__u32 class, class_mask;	/* (class,subclass,prog-if) triplet */
	__u32 unit;			/* Logical unit for attach */
	kernel_ulong_t driver_data;	/* Data private to the driver */
};


#define	to_vga_dev(n) container_of(n, struct vga_dev, dev)

struct vga_driver {
	struct list_head node;
	int type;
	char *name;
	int (*probe) (struct vga_dev *vdev, const struct vga_device_id *id);
	void (*remove) (struct vga_dev *dev, int users);	/* Device removed (NULL if not a hot-plug capable driver) */
	int  (*suspend) (struct vga_dev *vdev, u32 state);	/* Device suspended */	int  (*resume) (struct vga_dev *vdev);	                /* Device woken up */
	void (*notify_attach) (struct vga_dev *, int);
	void (*notify_detach) (struct vga_dev *, int);
	struct device_driver driver;
	struct vga_device_id *id_table;
};

#define to_vga_driver(drv) container_of(drv, struct vga_driver, driver)

extern struct bus_type vga_bus_type;

extern int vga_register_driver(struct vga_driver *drv);
extern void vg_unregister_driver(struct vga_driver *drv);
extern struct vga_driver *vga_dev_driver(const struct vga_dev *, int);
extern struct bus_type vga_bus_type;
extern struct vga_dev *vga_dev_get(struct vga_dev *);
extern void vga_dev_put(struct vga_dev *);
